/***

Author: Wilson Mui

Processing 3

Code adapted from Rodger Luo and George Legrady.

The purpose of this project is to visualize the monthly changes in checkout activity for 
Dewey classes 914-918 over a four year period. These categories refer to geographic
and travel books for specific regions. 

Pressing numbers 0-4 on keyboard will change the color scheme in use. Hover over 
a rectangle to view the number of checkouts for that category and month. 

***/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 40;

// width and height for each cell
float cellWidth, cellHeight;

// 2D matrix table related variables
Table table;

int numRows, numColumns;
float[][] dataMatrix;
float[][] x; 
float[][] y;

String[] titles = {"Europe", "Asia", "Africa", "North America", "South America"};
String[] months;
int[] checkouts;

String[] colorPath = {"reverse_gradient.PNG", "strong_gradient.PNG", "ytog.PNG", "lite.PNG", "dark.PNG"};
PImage colorBar;
boolean colorScheme = true;
float maxValue[] = {0.0, 0.0, 0.0, 0.0, 0.0};
float minValue[] = {0.0, 0.0, 0.0, 0.0, 0.0};
float minVal, maxVal = 0;


void setup() {
  //size(1280, 720); // setup the size of the window
  fullScreen();

  // initialize colorBar
  colorBar = loadImage(colorPath[0]);

  // Make the screen resizable.                     
  surface.setResizable(true);

  //Put data into Processing table from csv
  table = loadTable("maps_month_extended.csv");

  // Get row and column information
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  println("Rows: " + numRows + " Columns: " + numColumns);

  // Create a 2D array to receive data from the table 
  dataMatrix = new float[numRows][numColumns]; // doesn't need the first two columns 
  x = new float[numRows][numColumns];
  y = new float[numRows][numColumns];
  months = new String[numColumns];
  for (int i=1; i<numRows; i++) {
    for (int j=1; j<numColumns; j++) {
      dataMatrix[i][j] = table.getFloat(i, j);
      // find max for each row
      maxValue[i-1] = max(maxValue[i-1], dataMatrix[i][j]);
      // find min for each row
      minValue[i-1] = min(minValue[i-1], dataMatrix[i][j]);
      // get labels for months
      months[j] = table.getString(0, j);
      //get overall max and min values
      maxVal = max(maxVal, dataMatrix[i][j]);
      minVal = min(minVal, dataMatrix[i][j]);
    }
  }

  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/(numColumns-1);
  cellHeight = (height - constantVerticalMargin*2)/numRows;
}


void draw() {
  background(0);

  //Draw 2D Matrix
  for (int i=1; i<dataMatrix.length; i++) {
    for (int j=1; j<dataMatrix[0].length; j++) {

      stroke(255);
      strokeWeight(1);

      color c = colorBar.get((int)map(dataMatrix[i][j], minValue[i-1], maxValue[i-1], colorBar.width-2, 2), 
        colorBar.height/2);
      fill(c, map(dataMatrix[i][j], minVal, maxVal, 230, 255));

      x[i][j] = constantHorizontalMargin + cellWidth*j;
      y[i][j] = constantVerticalMargin + cellHeight*i;

      rect(x[i][j], y[i][j], cellWidth, cellHeight);
    }
  }
  for (int i=1; i<dataMatrix.length; i++) {
    for (int j=1; j<dataMatrix[0].length; j++) {
      if (mouseX > x[i][j] && mouseX < x[i][j]+cellWidth && mouseY > y[i][j] && mouseY < y[i][j]+cellHeight) {
        fill(255);
        text((int)dataMatrix[i][j], x[i][j]+12, y[i][j]+cellHeight/2);
      }
    }
  }

  //Draw month labels
  for (int i=1; i<numColumns; i++) {
    textSize(9);
    textAlign(CENTER, BOTTOM);
    fill(250);
    textFont(createFont("Arial", 8));
    //textLeading(9);
    text(months[i], constantHorizontalMargin + cellWidth*(i+1)-11, 
      constantVerticalMargin+cellHeight-5 );
  }

  //Draw category labels
  for (int i=1; i<numRows; i++) {
    textAlign(RIGHT, CENTER);
    fill(250);
    float fontSize = map(maxValue[i-1], minVal, maxVal, 10, 16);
    textSize(fontSize);
    text(titles[i-1], constantHorizontalMargin, constantVerticalMargin +cellHeight*(0.5+i));
  }

  //Draw Title
  textAlign(CENTER, TOP);
  textSize(20);
  fill(250);
  text("Checkout Activity of Geography + Travel Categories (2015-2018)", width/2, constantVerticalMargin+40);
}

void keyPressed() {
  if (key == '0') {
    colorScheme = true;
    colorBar = loadImage(colorPath[0]);
  } else if (key == '1') {
    colorScheme = true;
    colorBar = loadImage(colorPath[1]);
  } else if (key == '2') {
    colorScheme = true;
    colorBar = loadImage(colorPath[2]);
  } else if (key == '3') {
    colorScheme = true;
    colorBar = loadImage(colorPath[3]);
  } else if (key == '4') {
    colorScheme = true;
    colorBar = loadImage(colorPath[4]);
  }

  if (key == 's' || key == 'S') {
    saveFrame();
  }
}
